@@ -110,6 +110,7 @@ import jinja2
 
 # Side effects
 from rst2pdf.directives import aafigure  # noqa
+from rst2pdf.directives import contents  # noqa
 from rst2pdf.directives import oddeven  # noqa
 from rst2pdf.roles import counter as counter_role  # noqa
 from rst2pdf.roles import package as package_role  # noqa
@@ -161,6 +162,7 @@ class RstToPdf(object):
         floating_images=False,
         numbered_links=False,
         section_header_depth=2,
+        toc_depth=0,
         raw_html=False,
         strip_elements_with_classes=[],
     ):
@@ -225,6 +227,7 @@ class RstToPdf(object):
         self.show_frame = show_frame
         self.numbered_links = numbered_links
         self.section_header_depth = section_header_depth
+        self.toc_depth = toc_depth
         self.img_dir = os.path.join(self.PATH, 'images')
         self.raw_html = raw_html
         self.strip_elements_with_classes = strip_elements_with_classes
@@ -311,9 +314,9 @@ class RstToPdf(object):
         return sep + " "
 
     def styleToTags(self, style):
-        '''Takes a style name, returns a pair of opening/closing tags for it, like
+        """Takes a style name, returns a pair of opening/closing tags for it, like
         "<font face=helvetica size=14 color=red>". Used for inline
-        nodes (custom interpreted roles)'''
+        nodes (custom interpreted roles)"""
 
         try:
             s = self.styles[style]
@@ -341,9 +344,9 @@ class RstToPdf(object):
             return None
 
     def styleToFont(self, style):
-        '''Takes a style name, returns a font tag for it, like
+        """Takes a style name, returns a font tag for it, like
         "<font face=helvetica size=14 color=red>". Used for inline
-        nodes (custom interpreted roles)'''
+        nodes (custom interpreted roles)"""
 
         try:
             s = self.styles[style]
@@ -376,8 +379,8 @@ class RstToPdf(object):
 
     def bullet_for_node(self, node):
         """Takes a node, assumes it's some sort of
-           item whose parent is a list, and
-           returns the bullet text it should have"""
+        item whose parent is a list, and
+        returns the bullet text it should have"""
         b = ""
         t = 'item'
         if node.parent.get('start'):
@@ -534,13 +537,26 @@ class RstToPdf(object):
                 settings_overrides[
                     'strip_elements_with_classes'
                 ] = self.strip_elements_with_classes
-                self.doctree = docutils.core.publish_doctree(
-                    text, source_path=source_path, settings_overrides=settings_overrides
-                )
-                log.debug(self.doctree)
+                settings_overrides['exit_status_level'] = 3
+
+                try:
+                    self.doctree = docutils.core.publish_doctree(
+                        text,
+                        source_path=source_path,
+                        settings_overrides=settings_overrides,
+                    )
+                    log.debug(self.doctree)
+                except Exception as e:
+                    if log.isEnabledFor(logging.INFO):
+                        # Log exception with traceback if more detailed logging has been set
+                        log.exception('Error generating doctree')
+                    else:
+                        log.error(f"Error generating doctree: {e}")
+                    log.error("Cannot generate PDF, exiting")
+                    return 1
             else:
                 log.error('Error: createPdf needs a text or a doctree')
-                return
+                return 1
         else:
             self.doctree = doctree
 
@@ -559,7 +575,20 @@ class RstToPdf(object):
             sce = StripClassesAndElements(self.doctree)
             sce.apply()
 
-        elements = self.gen_elements(self.doctree)
+        if self.toc_depth == 0:
+            # use the `:depth:` option from `.. contents::`
+            self.toc_depth = contents.Contents.depth
+
+        try:
+            elements = self.gen_elements(self.doctree)
+        except Exception as e:
+            if log.isEnabledFor(logging.INFO):
+                # Log exception with traceback if more detailed logging has been set
+                log.exception('Error generating document elements')
+            else:
+                log.error(f"Error generating document elements: {e}")
+            log.error("Cannot generate PDF, exiting")
+            return 1
 
         # Find cover template, save it in cover_file
         jinja_env = jinja2.Environment(
@@ -655,6 +684,7 @@ class RstToPdf(object):
         while True:
             try:
                 log.info("Starting build")
+                self.elements = elements
                 # See if this *must* be multipass
                 pdfdoc.multiBuild(elements)
                 # Force a multibuild pass
@@ -719,8 +749,27 @@ class RstToPdf(object):
             except OSError:
                 pass
 
+        return 0
+
 
 class FancyDocTemplate(BaseDocTemplate):
+    def onProgress(self, typ, value):
+        global _counter
+        message = ''
+        if typ == 'SIZE_EST':
+            log.debug(f'Number of flowables: {value}')
+        elif typ == 'PROGRESS':
+            message = f'Flowable {value}'
+            # add class name for this flowable if we can
+            if hasattr(self.client, 'elements'):
+                if 0 <= value < len(self.client.elements):
+                    element = self.client.elements[value]
+                    message += f" {type(element)}"
+            log.debug(f'Page {_counter}: {message}')
+
+    def afterInit(self):
+        self.setProgressCallBack(self.onProgress)
+
     def afterFlowable(self, flowable):
 
         if isinstance(flowable, Heading):
@@ -791,16 +840,13 @@ class FancyDocTemplate(BaseDocTemplate):
                         flowables.insert(i, f)  # put split flowables back on the list
                 else:
                     if hasattr(f, '_postponed') and f._postponed > 4:
-                        ident = (
-                            "Flowable %s%s too large on page %d in frame %r%s of template %r"
-                            % (
-                                self._fIdent(f, 60, frame),
-                                doctemplate._fSizeString(f),
-                                self.page,
-                                self.frame.id,
-                                self.frame._aSpaceString(),
-                                self.pageTemplate.id,
-                            )
+                        ident = "Flowable %s%s too large on page %d in frame %r%s of template %r" % (
+                            self._fIdent(f, 60, frame),
+                            doctemplate._fSizeString(f),
+                            self.page,
+                            self.frame.id,
+                            self.frame._aSpaceString(),
+                            self.pageTemplate.id,
                         )
                         # leave to keep apart from the raise
                         raise LayoutError(ident)
@@ -863,9 +909,9 @@ class MyContainer(_Container, Flowable):
 
 
 class UnhappyOnce(IndexingFlowable):
-    '''An indexing flowable that is only unsatisfied once.
+    """An indexing flowable that is only unsatisfied once.
     If added to a story, it will make multiBuild run
-    at least two passes. Useful for ###Total###'''
+    at least two passes. Useful for ###Total###"""
 
     _unhappy = True
 
@@ -880,9 +926,9 @@ class UnhappyOnce(IndexingFlowable):
 
 
 class HeaderOrFooter(object):
-    """ A helper object for FancyPage (below)
-        HeaderOrFooter handles operations which are common
-        to both headers and footers
+    """A helper object for FancyPage (below)
+    HeaderOrFooter handles operations which are common
+    to both headers and footers
     """
 
     def __init__(self, items=None, isfooter=False, client=None):
@@ -985,8 +1031,7 @@ class HeaderOrFooter(object):
 
 
 class FancyPage(PageTemplate):
-    """ A page template that handles changing layouts.
-    """
+    """A page template that handles changing layouts."""
 
     def __init__(self, _id, _head, _foot, client):
         self.client = client
@@ -999,21 +1044,25 @@ class FancyPage(PageTemplate):
         PageTemplate.__init__(self, _id, [])
 
     def draw_background(self, which, canv):
-        ''' Draws a background and/or foreground image
-            on each page which uses the template.
+        """Draws a background and/or foreground image
+        on each page which uses the template.
 
-            Calculates the image one time, and caches
-            it for reuse on every page in the template.
+        Calculates the image one time, and caches
+        it for reuse on every page in the template.
 
-            How the background is drawn depends on the
-            --fit-background-mode option.
+        How the background is drawn depends on the
+        --fit-background-mode option.
 
-            If desired, we could add code to push it around
-            on the page, using stylesheets to align and/or
-            set the offset.
-        '''
+        If desired, we could add code to push it around
+        on the page, using stylesheets to align and/or
+        set the offset.
+        """
         uri = self.template[which]
-        info = self.image_cache.get(uri)
+        background_fit_mode = self.client.background_fit_mode
+        if 'background_fit_mode' in self.template:
+            background_fit_mode = self.template['background_fit_mode']
+
+        info = self.image_cache.get(f"{uri}{background_fit_mode}")
         if info is None:
             fname, _, _ = MyImage.split_uri(uri)
             if not os.path.exists(fname):
@@ -1021,27 +1070,33 @@ class FancyPage(PageTemplate):
                 log.error("Missing %s image file: %s", which, uri)
                 return
             try:
-                w, h, _ = MyImage.size_for_node(dict(uri=uri,), self.client)
+                w, h, _ = MyImage.size_for_node(
+                    dict(
+                        uri=uri,
+                    ),
+                    self.client,
+                )
             except ValueError:
                 # Broken image, return arbitrary stuff
                 uri = missing
-                w, h, = 100, 100
+                w, h, = (
+                    100,
+                    100,
+                )
 
             pw, ph = self.styles.pw, self.styles.ph
-            if self.client.background_fit_mode == 'center':
+            if background_fit_mode == 'center':
                 scale = min(1.0, 1.0 * pw / w, 1.0 * ph / h)
                 sw, sh = w * scale, h * scale
                 x, y = (pw - sw) / 2.0, (ph - sh) / 2.0
-            elif self.client.background_fit_mode == 'scale':
+            elif background_fit_mode == 'scale':
                 x, y = 0, 0
                 sw, sh = pw, ph
-            elif self.client.background_fit_mode == 'scale_width':
+            elif background_fit_mode == 'scale_width':
                 x, y = 0, 0
                 sw, sh = pw, h
             else:
-                log.error(
-                    'Unknown background fit mode: %s' % self.client.background_fit_mode
-                )
+                log.error('Unknown background fit mode: %s' % background_fit_mode)
                 # Do scale anyway
                 x, y = 0, 0
                 sw, sh = pw, ph
@@ -1054,7 +1109,7 @@ class FancyPage(PageTemplate):
     def is_left(self, page_num):
         """Default behavior is that the first page is on the left.
 
-           If the user has --first_page_on_right, the calculation is reversed.
+        If the user has --first_page_on_right, the calculation is reversed.
         """
         val = page_num % 2 == 1
         if self.client.first_page_on_right:
@@ -1064,7 +1119,7 @@ class FancyPage(PageTemplate):
     def beforeDrawPage(self, canv, doc):
         """Do adjustments to the page according to where we are in the document.
 
-           * Gutter margins on left or right as needed
+        * Gutter margins on left or right as needed
 
         """
 
@@ -1072,11 +1127,24 @@ class FancyPage(PageTemplate):
 
         styles = self.styles
         self.tw = styles.pw - styles.lm - styles.rm - styles.gm
+
         # What page template to use?
         tname = canv.__dict__.get('templateName', self.styles.firstTemplate)
+        if tname not in self.styles.pageTemplates:
+            log.error(f"Template '{tname}' is not defined")
+            sys.exit(1)
         self.template = self.styles.pageTemplates[tname]
         canv.templateName = tname
 
+        # Any background with fit mode?
+        background = canv.__dict__.get('background', None)
+        if background:
+            self.template = self.template.copy()
+            self.template['background'] = background
+            background_fit_mode = canv.__dict__.get('background_fit_mode', None)
+            if background_fit_mode:
+                self.template['background_fit_mode'] = background_fit_mode
+
         doct = getattr(canv, '_doctemplate', None)
         canv._doctemplate = None  # to make _listWrapOn work
 
@@ -1118,6 +1186,10 @@ class FancyPage(PageTemplate):
             self.draw_background('background', canv)
 
         self.frames = []
+        if 'frames' not in self.template:
+            log.error('No frames in template')
+            sys.exit(1)
+
         for frame in self.template['frames']:
             frame = frame[:]
             while len(frame) < 8:
@@ -1586,7 +1658,7 @@ def main(_args=None):
             PATH = abspath(dirname(sys.executable))
         else:
             PATH = abspath(dirname(__file__))
-        with open(join(PATH, 'styles', 'styles.style')) as fh:
+        with open(join(PATH, 'styles', 'styles.yaml')) as fh:
             print(fh.read())
         sys.exit(0)
 
@@ -1626,7 +1698,7 @@ def main(_args=None):
     if options.output:
         outfile = options.output
         if outfile == '-':
-            outfile = sys.stdout
+            outfile = sys.stdout.buffer
             options.compressed = False
             # we must stay quiet
             log.setLevel(logging.CRITICAL)
@@ -1637,7 +1709,7 @@ def main(_args=None):
             else:
                 outfile = filename + '.pdf'
         else:
-            outfile = sys.stdout
+            outfile = sys.stdout.buffer
             options.compressed = False
             # we must stay quiet
             log.setLevel(logging.CRITICAL)
@@ -1680,7 +1752,7 @@ def main(_args=None):
 
     add_extensions(options)
 
-    RstToPdf(
+    return_code = RstToPdf(
         stylesheets=options.style,
         language=options.language,
         header=options.header,
@@ -1720,14 +1792,16 @@ def main(_args=None):
     if close_infile:
         infile.close()
 
+    sys.exit(return_code)
+
 
 # Ugly hack that fixes Issue 335
 reportlab.lib.utils.ImageReader.__deepcopy__ = lambda self, *x: copy(self)
 
 
 def patch_digester():
-    ''' Patch digester so that we can get the same results when image
-filenames change'''
+    """Patch digester so that we can get the same results when image
+    filenames change"""
     import reportlab.pdfgen.canvas as canvas
 
     cache = {}
@@ -1803,23 +1877,30 @@ def add_extensions(options):
             prefix = os.getcwd()
         if prefix not in sys.path:
             sys.path.insert(0, prefix)
+
         log.info('Importing extension module %s', repr(modname))
+
         firstname = path_given and modname or (modname + '_r2p')
         _names = [firstname, modname]
-        try:
-            for _name in _names:
-                try:
-                    module = import_module(_name, 'rst2pdf')
-                except ImportError:
-                    continue
-        except ImportError as e:
-            if str(e).split()[-1].replace("'", '') not in [firstname, modname]:
-                raise
+        import_exc = None
+        for _name in _names:
+            try:
+                module = import_module(_name, 'rst2pdf')
+                break
+            except ImportError as e:
+                import_exc = e
+        else:
+            if not import_exc:
+                continue
+
+            if str(import_exc).split()[-1].replace("'", '') not in [firstname, modname]:
+                raise import_exc
+
             raise SystemExit(
-                '\nError: Could not find module %s '
-                'in sys.path [\n    %s\n]\nExiting...\n'
-                % (modname, ',\n    '.join(sys.path))
+                '\nError: Could not find module %s in sys.path [\n    %s\n]\n'
+                'Exiting...\n' % (modname, ',\n    '.join(sys.path))
             )
+
         if hasattr(module, 'install'):
             module.install(createpdf, options)
 
