@@ -127,10 +127,18 @@ class HandleTGroup(NodeHandler, docutils.nodes.tgroup):
                 colWidths.append(int(n['colwidth']))
 
         # colWidths are in no specific unit, really. Maybe ems.
-        # Convert them to %
-        colWidths = [int(x) for x in colWidths]
         tot = sum(colWidths)
-        colWidths = ["%s%%" % ((100.0 * w) / tot) for w in colWidths]
+        adjustedWidths = []
+        for x in colWidths:
+            # Convert them to %
+            w = 100 * x / tot
+            # Limit minimum width of a column as narrow columns cause strange "huge height" from reportlab if the cell
+            # padding is larger than the calculated width. Hack for #967
+            if w < 4:
+                w = 4
+            adjustedWidths.append("%s%%" % w)
+
+        colWidths = adjustedWidths
 
         if 'colWidths' in style.__dict__:
             colWidths[: len(style.colWidths)] = style.colWidths
@@ -236,6 +244,7 @@ class HandleTitle(HandleParagraph, docutils.nodes.title):
                     parent_id=parent_id,
                     node=node,
                     section_header_depth=client.section_header_depth,
+                    toc_depth=client.toc_depth,
                 )
             ]
             if client.depth <= client.breaklevel:
@@ -475,7 +484,8 @@ class HandleTopic(NodeHandler, docutils.nodes.topic):
             else:
                 node.elements = [
                     Paragraph(
-                        client.gen_pdftext(node.children[0]), cstyles['heading1'],
+                        client.gen_pdftext(node.children[0]),
+                        cstyles['heading1'],
                     ),
                     toc,
                 ]
@@ -698,7 +708,7 @@ class HandleListItem(NodeHandler, docutils.nodes.list_item):
 
 class HandleTransition(NodeHandler, docutils.nodes.transition):
     def gather_elements(self, client, node, style):
-        return [Separation()]
+        return [Separation(style=client.styles['separation'])]
 
 
 class HandleBlockQuote(NodeHandler, docutils.nodes.block_quote):
@@ -848,7 +858,8 @@ class HandleCaption(NodeHandler, docutils.nodes.caption):
     def gather_elements(self, client, node, style):
         return [
             Paragraph(
-                client.gather_pdftext(node), style=client.styles['figure-caption'],
+                client.gather_pdftext(node),
+                style=client.styles['figure-caption'],
             )
         ]
 
@@ -862,7 +873,8 @@ class HandleSidebar(NodeHandler, docutils.nodes.sidebar):
     def gather_elements(self, client, node, style):
         return [
             BoxedContainer(
-                client.gather_elements(node, style=None), client.styles['sidebar'],
+                client.gather_elements(node, style=None),
+                client.styles['sidebar'],
             )
         ]
 
@@ -1086,15 +1098,21 @@ class HandleAdmonition(
 class HandleMath(NodeHandler, docutils.nodes.math_block, docutils.nodes.math):
     def gather_elements(self, client, node, style):
         label = node.attributes.get('label')
-        return [Math(node.astext(), label, style.fontSize, style.textColor.rgb())]
+        return [
+            Math(
+                node.astext(),
+                label,
+                style,
+            )
+        ]
 
     def get_text(self, client, node, replaceEnt):
+        """Generate an image of the math equation for the inline role usage"""
         # get style for current node
-        sty = client.styles.styleForNode(node)
-        node_fontsize = sty.fontSize
-        node_color = '#' + sty.textColor.hexval()[2:]
+        style = client.styles.styleForNode(node)
         label = node.attributes.get('label')
-        mf = Math(node.astext(), label=label, fontsize=node_fontsize, color=node_color,)
+        mf = Math(node.astext(), label, style)
+        mf.fontsize = style.fontSize * 0.95
         w, h = mf.wrap(0, 0)
         descent = mf.descent()
         img = mf.genImage()
