@@ -15,7 +15,7 @@ from .log import log
 try:
     from matplotlib import mathtext
     from matplotlib.font_manager import FontProperties
-    from matplotlib.colors import ColorConverter
+    from matplotlib.colors import to_rgb
 
     HAS_MATPLOTLIB = True
 except ImportError:
@@ -32,91 +32,90 @@ def enclose(s):
 
 
 class Math(Flowable):
-    def __init__(self, s, label=None, fontsize=12, color='black'):
-        self.s = s
+    def __init__(self, s, label=None, style=None):
+        Flowable.__init__(self)
+        self.s = s.strip()
         self.label = label
-        self.fontsize = fontsize
-        self.color = color
+        self.fontsize = 10
+        self.color = (0, 0, 0)
+        self.hAlign = 'LEFT'
+        if style:
+            self.style = style
+            self.fontsize = style.fontSize
+            self.color = style.textColor.rgb()
+            self.hAlign = style.alignment
+
         if HAS_MATPLOTLIB:
-            self.parser = mathtext.MathTextParser("Pdf")
+            self.parser = mathtext.MathTextParser("Path")
         else:
             log.error(
                 "Math support not available,"
                 " some parts of this document will be rendered incorrectly."
                 " Install matplotlib."
             )
-        Flowable.__init__(self)
-        self.hAlign = 'CENTER'
+
+        if "\n" in s:
+            log.error("rst2pdf's math directive does not support multiple lines'")
 
     def wrap(self, aW, aH):
         if HAS_MATPLOTLIB:
             try:
-                (
-                    width,
-                    height,
-                    descent,
-                    glyphs,
-                    rects,
-                    used_characters,
-                ) = self.parser.parse(
+                (width, height, descent, _, _,) = self.parser.parse(
                     enclose(self.s), 72, prop=FontProperties(size=self.fontsize)
                 )
-                return width, height
-            except Exception:
+                return width, height + descent
+            except Exception as e:
+                log.error(f"Math error in wrap: {e}")
                 pass
-                # FIXME: report error
+
         return 10, 10
 
     def drawOn(self, canv, x, y, _sW=0):
         if _sW and hasattr(self, 'hAlign'):
-            from reportlab.lib.enums import TA_LEFT, TA_CENTER, TA_RIGHT
+            from reportlab.lib.enums import TA_LEFT, TA_CENTER, TA_RIGHT, TA_JUSTIFY
 
             a = self.hAlign
             if a in ('CENTER', 'CENTRE', TA_CENTER):
                 x = x + 0.5 * _sW
             elif a in ('RIGHT', TA_RIGHT):
                 x = x + _sW
-            elif a not in ('LEFT', TA_LEFT):
+            elif a not in ('LEFT', TA_LEFT, TA_JUSTIFY):
                 raise ValueError("Bad hAlign value " + str(a))
         height = 0
         if HAS_MATPLOTLIB:
             global fonts
             canv.saveState()
-            canv.translate(x, y)
             try:
-                (
-                    width,
-                    height,
-                    descent,
-                    glyphs,
-                    rects,
-                    used_characters,
-                ) = self.parser.parse(
+                (width, height, descent, glyphs, rects,) = self.parser.parse(
                     enclose(self.s), 72, prop=FontProperties(size=self.fontsize)
                 )
-                for ox, oy, fontname, fontsize, num, symbol_name in glyphs:
+                canv.translate(x, y + descent)
+
+                for font, fontsize, num, ox, oy in glyphs:
+                    fontname = font.fname
                     if fontname not in fonts:
                         fonts[fontname] = fontname
                         pdfmetrics.registerFont(TTFont(fontname, fontname))
                     canv.setFont(fontname, fontsize)
-                    col_conv = ColorConverter()
-                    rgb_color = col_conv.to_rgb(self.color)
+                    rgb_color = to_rgb(self.color)
                     canv.setFillColorRGB(rgb_color[0], rgb_color[1], rgb_color[2])
                     canv.drawString(ox, oy, chr(num))
 
                 canv.setLineWidth(0)
                 canv.setDash([])
                 for ox, oy, width, height in rects:
-                    canv.rect(ox, oy + 2 * height, width, height, fill=1)
-            except Exception:
-                # FIXME: report error
-                col_conv = ColorConverter()
-                rgb_color = col_conv.to_rgb(self.color)
+                    canv.rect(ox, oy + height, width, height, fill=1)
+            except Exception as e:
+                log.error(f"Math error: {e}")
+                log.exception("Math error!")
+                canv.translate(x, y)
+                rgb_color = to_rgb(self.color)
                 canv.setFillColorRGB(rgb_color[0], rgb_color[1], rgb_color[2])
                 canv.drawString(0, 0, self.s)
             canv.restoreState()
         else:
             canv.saveState()
+            canv.translate(x, y)
             canv.drawString(x, y, self.s)
             canv.restoreState()
         if self.label:
@@ -127,7 +126,7 @@ class Math(Flowable):
         """Return the descent of this flowable,
         useful to align it when used inline."""
         if HAS_MATPLOTLIB:
-            width, height, descent, glyphs, rects, used_characters = self.parser.parse(
+            width, height, descent, glyphs, rects = self.parser.parse(
                 enclose(self.s), 72, prop=FontProperties(size=self.fontsize)
             )
             return descent
@@ -138,12 +137,12 @@ class Math(Flowable):
 
         Required so we can put inline math in paragraphs.
         Returns the file name.
-        The file is caller's responsability.
+        The file is caller's responsibility.
 
         """
 
         dpi = 72
-        scale = 10
+        scale = 72
 
         try:
             import Image
@@ -159,33 +158,33 @@ class Math(Flowable):
         if not HAS_MATPLOTLIB:
             img = Image.new('RGBA', (120, 120), (255, 255, 255, 0))
         else:
-            width, height, descent, glyphs, rects, used_characters = self.parser.parse(
+            (width, height, descent, glyphs, rects,) = self.parser.parse(
                 enclose(self.s), dpi, prop=FontProperties(size=self.fontsize)
             )
             img = Image.new(
-                'RGBA', (int(width * scale), int(height * scale)), (255, 255, 255, 0)
+                'RGBA',
+                (int(width * scale), int((height + descent) * scale)),
+                (255, 255, 255, 0),
             )
+            log.critical(f"descent = {descent}")
             draw = ImageDraw.Draw(img)
-            for ox, oy, fontname, fontsize, num, symbol_name in glyphs:
-                font = ImageFont.truetype(fontname, int(fontsize * scale))
-                tw, th = draw.textsize(chr(num), font=font)
-                # No, I don't understand why that 4 is there.
-                # As we used to say in the pure math
-                # department, that was a numerical solution.
-                col_conv = ColorConverter()
-                fc = col_conv.to_rgb(self.color)
+            for font, fontsize, num, ox, oy in glyphs:
+                fontname = font.fname
+                image_font = ImageFont.truetype(fontname, int(fontsize * scale))
+                log.critical(f"chr(num) = {chr(num)}")
+                fc = to_rgb(self.color)
                 rgb_color = (int(fc[0] * 255), int(fc[1] * 255), int(fc[2] * 255))
                 draw.text(
-                    (ox * scale, (height - oy - fontsize + 4) * scale),
+                    (ox * scale, (height + 1 - oy - fontsize) * scale),
                     chr(num),
-                    font=font,
+                    font=image_font,
                     fill=rgb_color,
                 )
             for ox, oy, w, h in rects:
                 x1 = ox * scale
                 x2 = x1 + w * scale
-                y1 = (height - oy) * scale
-                y2 = y1 + h * scale
+                y1 = (height - 1 - oy) * scale
+                y2 = y1 + (h * scale)
                 draw.rectangle([x1, y1, x2, y2], (0, 0, 0))
 
         fh, fn = tempfile.mkstemp(suffix=".png")
