@@ -17,6 +17,7 @@ import reportlab.lib.units as units
 from reportlab.pdfbase.ttfonts import TTFont
 from reportlab.pdfbase import pdfmetrics
 import reportlab.rl_config
+import yaml
 
 from . import findfonts
 from .log import log
@@ -26,14 +27,14 @@ unit_separator = re.compile('(-?[0-9.]*)')
 
 
 class StyleSheet(object):
-    '''Class to handle a collection of stylesheets'''
+    """Class to handle a collection of stylesheets"""
 
     @staticmethod
     def stylepairs(data):
-        ''' Allows pairs of style information to be expressed
-            in canonical reportlab list of two-item list/tuple,
-            or in a more human-readable dictionary.
-        '''
+        """Allows pairs of style information to be expressed
+        in canonical reportlab list of two-item list/tuple,
+        or in a more human-readable dictionary.
+        """
         styles = data.get('styles', {})
         try:
             stylenames = list(styles.keys())
@@ -68,6 +69,9 @@ class StyleSheet(object):
 
     def __init__(self, flist, font_path=None, style_path=None, def_dpi=300):
         log.info('Using stylesheets: %s' % ','.join(flist))
+
+        self.suppress_undefined_style_warning = False
+
         # find base path
         if hasattr(sys, 'frozen'):
             self.PATH = os.path.abspath(os.path.dirname(sys.executable))
@@ -79,8 +83,8 @@ class StyleSheet(object):
         # but the two default stylesheets will always
         # be loaded first
         flist = [
-            os.path.join(self.PATH, 'styles', 'styles.style'),
-            os.path.join(self.PATH, 'styles', 'default.style'),
+            os.path.join(self.PATH, 'styles', 'styles.yaml'),
+            os.path.join(self.PATH, 'styles', 'default.yaml'),
         ] + flist
 
         self.def_dpi = def_dpi
@@ -230,34 +234,8 @@ class StyleSheet(object):
 
             for font in embedded:
                 try:
-                    # Just a font name, try to embed it
-                    if isinstance(font, str):
-                        # See if we can find the font
-                        fname, pos = findfonts.guessFont(font)
-                        if font in embedded_fontnames:
-                            pass
-                        else:
-                            fontList = findfonts.autoEmbed(font)
-                            if fontList:
-                                embedded_fontnames.append(font)
-                        if not fontList:
-                            if (fname, pos) in embedded_fontnames:
-                                fontList = None
-                            else:
-                                fontList = findfonts.autoEmbed(fname)
-                        if fontList is not None:
-                            self.embedded += fontList
-                            # Maybe the font we got is not called
-                            # the same as the one we gave
-                            # so check that out
-                            suff = ["", "-Oblique", "-Bold", "-BoldOblique"]
-                            if not fontList[0].startswith(font):
-                                # We need to create font aliases, and use them
-                                for fname, aliasname in zip(
-                                    fontList, [font + suffix for suffix in suff],
-                                ):
-                                    self.fontsAlias[aliasname] = fname
-                        continue
+                    # (removed the feature that supported a string here, this isn't
+                    # documented or supported)
 
                     # Each "font" is a list of four files, which will be
                     # used for regular / bold / italic / bold+italic
@@ -280,14 +258,15 @@ class StyleSheet(object):
                     if font[0].lower().endswith('.ttf'):  # A True Type font
                         for variant in font:
                             location = self.findFont(variant)
-                            pdfmetrics.registerFont(
-                                TTFont(str(variant.split('.')[0]), location)
-                            )
+                            # strip extension and leading path to get the name
+                            # to register the font under
+                            filename = os.path.basename(variant)
+                            fontname = str(filename.split('.')[0])
+                            pdfmetrics.registerFont(TTFont(fontname, location))
                             log.info(
-                                'Registering font: %s from %s'
-                                % (str(variant.split('.')[0]), location)
+                                'Registering font: %s from %s' % (fontname, location)
                             )
-                            self.embedded.append(str(variant.split('.')[0]))
+                            self.embedded.append(fontname)
 
                         # And map them all together
                         regular, bold, italic, bolditalic = [
@@ -317,16 +296,15 @@ class StyleSheet(object):
                             fname = font[0]
                         else:
                             fname = font
-                        log.error(
+                        log.critical(
                             "Error processing font %s: %s",
                             os.path.splitext(fname)[0],
                             str(e),
                         )
-                        log.error("Registering %s as Helvetica alias", fname)
-                        self.fontsAlias[fname] = 'Helvetica'
+                        sys.exit(1)
                     except Exception as e:
                         log.critical("Error processing font %s: %s", fname, str(e))
-                        continue
+                        sys.exit(1)
 
         # Go though all styles in all stylesheets and find all fontNames.
         # Then decide what to do with them
@@ -362,12 +340,10 @@ class StyleSheet(object):
                         # See if we can find the font
                         fname, pos = findfonts.guessFont(style[key])
 
-                        if style[key] in embedded_fontnames:
-                            pass
-                        else:
-                            fontList = findfonts.autoEmbed(style[key])
-                            if fontList:
-                                embedded_fontnames.append(style[key])
+                        fontList = findfonts.autoEmbed(style[key])
+                        if style[key] not in embedded_fontnames and fontList:
+                            embedded_fontnames.append(style[key])
+
                         if not fontList:
                             if (fname, pos) in embedded_fontnames:
                                 fontList = None
@@ -375,6 +351,7 @@ class StyleSheet(object):
                                 fontList = findfonts.autoEmbed(fname)
                             if fontList:
                                 embedded_fontnames.append((fname, pos))
+
                         if fontList:
                             self.embedded += fontList
                             # Maybe the font we got is not called
@@ -384,7 +361,8 @@ class StyleSheet(object):
                                 # We need to create font aliases, and use them
                                 basefname = style[key].split('-')[0]
                                 for fname, aliasname in zip(
-                                    fontList, [basefname + suffix for suffix in suff],
+                                    fontList,
+                                    [basefname + suffix for suffix in suff],
                                 ):
                                     self.fontsAlias[aliasname] = fname
                                 style[key] = self.fontsAlias[basefname + suff[pos]]
@@ -538,9 +516,9 @@ class StyleSheet(object):
                 self.StyleSheet.add(ParagraphStyle(**s))
 
         self.emsize = self['base'].fontSize
-        # Make stdFont the basefont, for Issue 65
+        # Set the basefont, for Issue 65
         reportlab.rl_config.canvas_basefontname = self['base'].fontName
-        # Make stdFont the default font for table cell styles (Issue 65)
+        # Set a default font for table cell styles (Issue 65)
         reportlab.platypus.tables.CellStyle.fontname = self['base'].fontName
 
     def __getitem__(self, key):
@@ -555,27 +533,37 @@ class StyleSheet(object):
         if key in self.StyleSheet:
             return self.StyleSheet[key]
         else:
-            if key.startswith('pygments'):
-                log.info(
-                    "Using undefined style '%s'" ", aliased to style 'code'." % key
-                )
-                newst = copy(self.StyleSheet['code'])
+            # Sphinx 4 doesn't prepend "pygments-" to its style names when using pygments,
+            # so look to see if we have a pygemts style for this style name
+            pygments_key = "pygments-" + key
+            if pygments_key in self.StyleSheet:
+                return self.StyleSheet[pygments_key]
             else:
-                log.warning(
-                    "Using undefined style '%s'" ", aliased to style 'normal'." % key
-                )
-                newst = copy(self.StyleSheet['normal'])
-            newst.name = key
-            self.StyleSheet.add(newst)
-            return newst
+                if key.startswith('pygments'):
+                    if not self.suppress_undefined_style_warning:
+                        log.info(
+                            "Using undefined style '%s'"
+                            ", aliased to style 'code'." % key
+                        )
+                    newst = copy(self.StyleSheet['code'])
+                else:
+                    if not self.suppress_undefined_style_warning:
+                        log.warning(
+                            "Using undefined style '%s'"
+                            ", aliased to style 'normal'." % key
+                        )
+                    newst = copy(self.StyleSheet['normal'])
+                newst.name = key
+                self.StyleSheet.add(newst)
+                return newst
 
     def readSheets(self, flist):
-        ''' Read in the stylesheets.  Return a list of
-            (sheetdata, sheetname) tuples.
+        """Read in the stylesheets.  Return a list of
+        (sheetdata, sheetname) tuples.
 
-            Orders included sheets in front
-            of including sheets.
-        '''
+        Orders included sheets in front
+        of including sheets.
+        """
 
         # Process from end of flist
         flist.reverse()
@@ -609,7 +597,17 @@ class StyleSheet(object):
         fname = self.findStyle(ssname)
         if fname:
             try:
-                return rson_loads(open(fname).read())
+                # TODO no longer needed when we drop support for rson
+                # Is it an older rson/json stylesheet with .style extension?
+                root_ext = os.path.splitext(fname)
+                if root_ext[1] == ".style":
+                    log.warning(
+                        'Stylesheet "%s" in outdated format, recommend converting to YAML'
+                        % (fname)
+                    )
+                    return rson_loads(open(fname).read())
+                # Otherwise assume yaml/yml
+                return yaml.safe_load(open(fname).read())
             except ValueError as e:  # Error parsing the JSON data
                 log.critical('Error parsing stylesheet "%s": %s' % (fname, str(e)))
             except IOError as e:  # Error opening the ssheet
@@ -634,7 +632,7 @@ class StyleSheet(object):
                         return tfn
             return None
 
-        for ext in ['', '.style', '.json']:
+        for ext in ['', '.yaml', '.yml', '.style', '.json']:
             result = innerFind(self.StyleSearchPath, fn + ext)
             if result:
                 break
@@ -686,9 +684,19 @@ class StyleSheet(object):
             self['table-heading'].alignment
         ]
         return [
-            ('BACKGROUND', (0, 0), (-1, rows - 1), self['table-heading'].backColor,),
+            (
+                'BACKGROUND',
+                (0, 0),
+                (-1, rows - 1),
+                self['table-heading'].backColor,
+            ),
             ('ALIGN', (0, 0), (-1, rows - 1), alignment),
-            ('TEXTCOLOR', (0, 0), (-1, rows - 1), self['table-heading'].textColor,),
+            (
+                'TEXTCOLOR',
+                (0, 0),
+                (-1, rows - 1),
+                self['table-heading'].textColor,
+            ),
             (
                 'FONT',
                 (0, 0),
@@ -706,7 +714,7 @@ class StyleSheet(object):
         return adjustUnits(v, total, self.def_dpi, default_unit, emsize=self.emsize)
 
     def combinedStyle(self, styles):
-        '''Given a list of style names, it merges them (the existing ones)
+        """Given a list of style names, it merges them (the existing ones)
         and returns a new style.
 
         The styles that don't exist are silently ignored.
@@ -715,7 +723,7 @@ class StyleSheet(object):
         style will be called 'merged_style1_style2'.
 
         The styles that are *later* in the list will have priority.
-        '''
+        """
 
         validst = [x for x in styles if x in self.StyleSheet]
         newname = '_'.join(['merged'] + validst)
@@ -850,8 +858,8 @@ validCommands = {
 
 
 def validateCommands(commands):
-    '''Given a list of reportlab's table commands, it fixes some common errors
-    and/or removes commands that can't be fixed'''
+    """Given a list of reportlab's table commands, it fixes some common errors
+    and/or removes commands that can't be fixed"""
 
     fixed = []
 
@@ -912,11 +920,11 @@ def validateCommands(commands):
 
 
 class CallableStyleSheet(str):
-    ''' Useful for programmatically generated stylesheets.
-        A generated stylesheet is a callable string (name),
-        which returns the pre-digested stylesheet data
-        when called.
-    '''
+    """Useful for programmatically generated stylesheets.
+    A generated stylesheet is a callable string (name),
+    which returns the pre-digested stylesheet data
+    when called.
+    """
 
     def __new__(cls, name, value=''):
         self = str.__new__(cls, name)
