@@ -24,6 +24,26 @@ OUTPUT_DIR = os.path.join(ROOT_DIR, 'output')
 REFERENCE_DIR = os.path.join(ROOT_DIR, 'reference')
 
 
+def can_run(command_list):
+    def _can_run():
+        try:
+            p = subprocess.Popen(
+                command_list,
+                stdout=subprocess.PIPE,
+                stdin=subprocess.PIPE,
+                stderr=subprocess.PIPE,
+            )
+            p.terminate()
+        except FileNotFoundError:
+            return False
+        return True
+
+    return _can_run
+
+
+check_dependency = {'plantuml': can_run(["plantuml", "-pipe"])}
+
+
 def _get_metadata(pdf):
     metadata = pdf.metadata
 
@@ -37,7 +57,7 @@ def _get_pages(pdf):
     pages = []
 
     for page in pdf.pages():
-        pages.append(page.getText('blocks'))
+        pages.append(page.get_text('blocks'))
 
     return pages
 
@@ -77,15 +97,34 @@ def compare_pdfs(path_a, path_b):
 
     def fuzzy_coord_diff(coord_a, coord_b):
         diff = abs(coord_a - coord_b)
-        assert diff / max(coord_a, coord_b) < 0.04  # allow an arbitrary diff
+        threshold = 3  # 3 px is approximately 1.06mm
+        assert (
+            diff < threshold
+        ), "Coordinates of the last printed block differs from the reference"
 
     def fuzzy_string_diff(string_a, string_b):
+        a_is_image = string_a.startswith("<image: DeviceRGB")
+        b_is_image = string_b.startswith("<image: DeviceRGB")
+        if a_is_image and b_is_image:
+            # We can't necessarily control the image metadata text in the block (e.g. from plantuml), so we do not
+            # check it.
+            return
+
         words_a = string_a.split()
-        words_b = string_a.split()
-        assert words_a == words_b
+        words_b = string_b.split()
+        assert (
+            words_a == words_b
+        ), "Text of the last printed block differs from the reference"
 
     assert len(pages_a) == len(pages_b)
+    page_no = 0
     for page_a, page_b in zip(pages_a, pages_b):
+        page_no = page_no + 1
+        print(f"++ Page {page_no} ++")
+        print(f"page_a: {page_a}")
+        print(f"page_b: {page_b}")
+        print("number of blocks in page_a: %s" % len(page_a))
+        print("number of blocks in page_b: %s" % len(page_b))
         assert len(page_a) == len(page_b)
         for block_a, block_b in zip(page_a, page_b):
             # each block has the following format:
@@ -95,6 +134,8 @@ def compare_pdfs(path_a, path_b):
             # block_type and block_no should remain unchanged, but it's
             # possible for the blocks to move around the document slightly and
             # the text refold without breaking entirely
+            print(f"block_a: {block_a}")
+            print(f"block_b: {block_b}")
             fuzzy_coord_diff(block_a[0], block_b[0])
             fuzzy_coord_diff(block_a[1], block_b[1])
             fuzzy_coord_diff(block_a[2], block_b[2])
@@ -113,10 +154,10 @@ class File(pytest.File):
             return cls(parent=parent, fspath=fspath)
 
 
-class TxtFile(File):
+class RstFile(File):
     def collect(self):
         name = os.path.splitext(self.fspath.basename)[0]
-        yield TxtItem.from_parent(parent=self, name=name)
+        yield RstItem.from_parent(parent=self, name=name)
 
 
 class SphinxFile(File):
@@ -138,7 +179,8 @@ class Item(pytest.Item):
 
     def _fail(self, msg, output=None):
         pytest.fail(
-            f'{msg}:\n\n{output.decode("utf-8")}' if output else msg, pytrace=False,
+            f'{msg}:\n\n{output.decode("utf-8")}' if output else msg,
+            pytrace=False,
         )
 
     def runtest(self):
@@ -153,13 +195,40 @@ class Item(pytest.Item):
 
             pytest.skip(ignore_reason)
 
+        # if '.depends' file is present, check if all dependencies are
+        # satisfied, otherwise skip test
+
+        depends_file = os.path.join(INPUT_DIR, self.name + '.depends')
+        if os.path.exists(depends_file):
+            with open(depends_file) as fh:
+                for line in fh:
+                    dep = line.rstrip('\n')
+                    try:
+                        if not check_dependency[dep]():
+                            pytest.skip("Unmet dependency for test: %s" % line)
+                    except KeyError:
+                        pytest.skip("Unknown test dependency: %s" % line)
+
         # run the actual test
 
         retcode, output = self._build()
 
         # verify results
 
-        if retcode:
+        retcode_file = os.path.join(INPUT_DIR, self.name + '.retcode')
+        if os.path.exists(retcode_file):
+            with open(retcode_file) as f:
+                first_line = f.readline()
+                expected_retcode = int(first_line)
+                if expected_retcode == retcode:
+                    return
+                else:
+                    self._fail(
+                        'Exit code of %d did not match expected %d'
+                        % (retcode, expected_retcode),
+                        output,
+                    )
+        elif retcode > 0:
             self._fail('Call failed with %d' % retcode, output)
 
         no_pdf = os.path.exists(os.path.join(INPUT_DIR, self.name + '.nopdf'))
@@ -196,7 +265,8 @@ class Item(pytest.Item):
 
         if len(reference_files) != len(output_files):
             self._fail(
-                'Mismatch between number of files expected and generated', output,
+                'Mismatch between number of files expected and generated',
+                output,
             )
 
         reference_files.sort()
@@ -219,9 +289,9 @@ class Item(pytest.Item):
         return self.fspath, 0, self.name
 
 
-class TxtItem(Item):
+class RstItem(Item):
     def _build(self):
-        input_ref = self.name + '.txt'
+        input_ref = self.name + '.rst'
         output_pdf = os.path.join(OUTPUT_DIR, self.name + '.pdf')
         output_log = os.path.join(OUTPUT_DIR, self.name + '.log')
 
@@ -239,15 +309,18 @@ class TxtItem(Item):
             with open(cli_file) as fh:
                 cmd += shlex.split(fh.read())
 
-        style_file = os.path.join(INPUT_DIR, self.name + '.style')
+        style_file = os.path.join(INPUT_DIR, self.name + '.yaml')
         if os.path.exists(style_file):
             cmd += ['-s', os.path.basename(style_file)]
 
-        cmd += ['-o', output_pdf]
+        if '-o' not in cmd:
+            cmd += ['-o', output_pdf]
 
         try:
             output = subprocess.check_output(
-                cmd, cwd=INPUT_DIR, stderr=subprocess.STDOUT,
+                cmd,
+                cwd=INPUT_DIR,
+                stderr=subprocess.STDOUT,
             )
             retcode = 0
         except subprocess.CalledProcessError as exc:
@@ -257,8 +330,28 @@ class TxtItem(Item):
         with open(output_log, 'wb') as fh:
             fh.write(output)
 
+        expected_log_file = os.path.join(
+            os.path.join(INPUT_DIR, self.name + '.expected_log')
+        )
         output_file = os.path.join(OUTPUT_DIR, self.name + '.pdf')
         no_pdf = os.path.exists(os.path.join(INPUT_DIR, self.name + '.nopdf'))
+
+        if os.path.exists(expected_log_file):
+            # look for each line in expected_log_file within the output log
+            output_string = output.decode("utf-8")
+            with open(expected_log_file) as expected_log_file:
+                for message in expected_log_file:
+                    message = message.strip()
+                    if message and message not in output_string:
+                        self._fail(
+                            'Log message %s was not found' % message,
+                            output,
+                        )
+            if no_pdf:
+                # As we are looking for a log entry, we ignore any created pdf file if a .nopdf
+                # file exists. That is, it doesn't matter if the file was created or not.
+                return retcode, output
+
         if not os.path.exists(output_file) and not no_pdf:
             self._fail(
                 'File %r was not generated' % (os.path.relpath(output_file, ROOT_DIR),),
@@ -295,7 +388,9 @@ class SphinxItem(Item):
 
         try:
             output = subprocess.check_output(
-                cmd, cwd=INPUT_DIR, stderr=subprocess.STDOUT,
+                cmd,
+                cwd=INPUT_DIR,
+                stderr=subprocess.STDOUT,
             )
             retcode = 0
         except subprocess.CalledProcessError as exc:
@@ -329,8 +424,8 @@ def pytest_collect_file(parent, path):
 
     parent_dir = os.path.split(path.dirname)[-1]
 
-    if path.ext == '.txt' and parent_dir == 'input':
-        return TxtFile.from_parent(parent=parent, fspath=path)
+    if path.ext == '.rst' and parent_dir == 'input':
+        return RstFile.from_parent(parent=parent, fspath=path)
     elif path.basename == 'conf.py' and parent_dir.startswith('sphinx'):
         return SphinxFile.from_parent(parent=parent, fspath=path)
 
