@@ -48,8 +48,12 @@ pdf_documents = [
         },
     ),
 ]
+pdf_stylesheets = ['sphinx', 'sphinx-issue364']
 pdf_use_index = False
 pdf_use_modindex = False
 pdf_use_coverpage = False
 pdf_invariant = True
 pdf_real_footnotes = True
+
+# Set a consistent date for the cover page
+today = 'April 29, 2018'
\ No newline at end of file
